#include "id3ed.h"

void
copy_string(const char *name, char *ptr, size_t size, const char *arg)
{
	size_t len = strlen(arg);
	if (len > size) {
		error(0, 0, "%s truncated", name);
		len = size;
	}
	memcpy(ptr, arg, len);
}
		

void
verify_mp3(FILE *fp, const char *name)
{
	char str[3];
	unsigned short sig = 0;
	int c = 0;

	if (fread(str, sizeof(str), 1, fp) != 1) 
		error(1, errno, "error reading %s", name);
	if (memcmp(str, "ID3", 3) == 0) {
		error(0, 0, "%s: warning: ID3 header found",
		      name);
		return;
	}
	sig = ((unsigned char)str[0]) << 8 | ((unsigned char)str[1]);

	switch (sig & 0xFFFE) {
	case 0xFFFA: /* MPEG ADTS, layer III, v1 */
	case 0xFFF2: /* MPEG ADTS, layer III, v2 */
	case 0xFFFE: /* MPEG ADTS, layer III,  v2.5 */
		break;

	default:
		error(1, 0, "%s: not an mp3 file", name);
	}
}

void
read_id3(FILE *fp, const char *name, struct id3_block *blk)
{
	if (fseek(fp, -sizeof(blk[0]), SEEK_END)) 
		error(1, errno, "%s: seek error", name);
	if (fread(blk, sizeof(blk[0]), 1, fp) != 1) 
		error(1, errno, "%s: read error: %s", name);
}

#define ID3_MERGE_FIELD(new,old,fld) do {				\
		if (new->fld[0] == 0 && old->fld)			\
			memcpy(new->fld, old->fld, sizeof(new->fld));	\
	} while (0)

void
id3_merge(struct id3_block *id, const struct id3_block *old)
{
	ID3_MERGE_FIELD(id,old,title);
	ID3_MERGE_FIELD(id,old,artist);
	ID3_MERGE_FIELD(id,old,album);
	ID3_MERGE_FIELD(id,old,year);
	ID3_MERGE_FIELD(id,old,comment);
}

void
set_id3(const char *name, const struct id3_block *id)
{
	int rc;
	struct id3_block old, new;
	FILE *fp = fopen(name, "r+");
	if (!fp) 
		error(1, errno, "cannot open file %s", name);

	verify_mp3(fp, name);
	read_id3(fp, name, &old);

	memcpy(&new, id, sizeof(new));
	if (memcmp(old.header, ID3_TAG, ID3_HEADER_LEN) == 0) {
		id3_merge(&new, &old);
		rc = fseek(fp, -sizeof(id[0]), SEEK_END);
	} else
		rc = fseek(fp, 0, SEEK_END);
	if (rc) 
		error(1, errno, "%s: seek error", name);

	if (fwrite(&new, sizeof(new), 1, fp) != 1) 
		error(1, errno, "%s: write error", name);
			
	fclose(fp);		
}

void
query_id3(const char *name, const struct id3_block *blk_unused)
{
	struct id3_block id;
	FILE *fp = fopen(name, "r");
	if (!fp) 
		error(1, errno, "cannot open file %s", name);

	verify_mp3(fp, name);
	read_id3(fp, name, &id);
	fclose(fp);
	if (memcmp(id.header, ID3_TAG, ID3_HEADER_LEN))
		return;
	printf("title: %-.30s\n", id.title);
	printf("artist: %-.30s\n", id.artist);
	printf("album: %-.30s\n", id.album);
	printf("year: %-.4s\n", id.year);
	printf("comment: %-.28s\n", id.comment);
	printf("genre: %s\n", id3v1_n_to_genre(id.genre[0]));
	if (id.pad[0] == 0)
		printf("track: %d\n", id.track[0]);
}

#define ID3_COPY(blk, fld, arg)					\
	copy_string(#fld, blk.fld, sizeof(blk.fld), arg)

void
del_id3(const char *name, const struct id3_block *blk_unused)
{
}

#define MODE_QUERY  0
#define MODE_MOD    1
#define MODE_DELETE 2

void (*id3_mode[])(const char *, const struct id3_block *) = {
	query_id3,
	set_id3,
	del_id3
};

struct id3_block id;
int mode = MODE_QUERY;

#include "cmdline.h"

int
main(int argc, char **argv)
{
	int c, n;
	
	set_program_name(argv[0]);
	memset(&id, 0, sizeof(id));
	memcpy(id.header, ID3_TAG, ID3_HEADER_LEN);

	get_options(argc, argv);
		
	argc -= optind;
	argv += optind;

	if (argc == 0) 
		error(1, 0, "no files");

	while (argc--) 
		id3_mode[mode](*argv++, &id);
	
	exit(0);
}
